@@ -69,31 +69,23 @@ DEALINGS IN THE SOFTWARE.
 #include <kwindowsystem.h>
 #include <kxmessages.h>
 #include <X11/Xlib.h>
-#include <X11/Xutil.h>
 #include <fixx11h.h>
 #endif
 
-static const char *const NET_STARTUP_MSG = "_NET_STARTUP_INFO";
-static const char *const NET_STARTUP_WINDOW = "_NET_STARTUP_ID";
+static const char NET_STARTUP_MSG[] = "_NET_STARTUP_INFO";
+static const char NET_STARTUP_WINDOW[] = "_NET_STARTUP_ID";
 // DESKTOP_STARTUP_ID is used also in kinit/wrapper.c ,
 // kdesu in both kdelibs and kdebase and who knows where else
-static const char *const NET_STARTUP_ENV = "DESKTOP_STARTUP_ID";
-
-static bool auto_app_started_sending = true;
+static const char NET_STARTUP_ENV[] = "DESKTOP_STARTUP_ID";
 
 static QByteArray s_startup_id;
 
 static long get_num(const QString &item_P);
-static unsigned long get_unum(const QString &item_P);
 static QString get_str(const QString &item_P);
 static QByteArray get_cstr(const QString &item_P);
 static QStringList get_fields(const QString &txt_P);
 static QString escape_str(const QString &str_P);
 
-#if KWINDOWSYSTEM_HAVE_X11
-static Atom utf8_string_atom = None;
-#endif
-
 class KStartupInfo::Data
     : public KStartupInfoData
 {
@@ -114,7 +106,7 @@ struct KStartupInfoId::Private {
 
 struct KStartupInfoData::Private {
     Private() : desktop(0), wmclass(""), hostname(""),
-        silent(KStartupInfoData::Unknown), timestamp(~0U), screen(-1), xinerama(-1), launched_by(0) {}
+        silent(KStartupInfoData::Unknown), screen(-1), xinerama(-1), launched_by(0) {}
 
     QString to_text() const;
     void remove_pid(pid_t pid);
@@ -128,7 +120,6 @@ struct KStartupInfoData::Private {
     QByteArray wmclass;
     QByteArray hostname;
     KStartupInfoData::TriState silent;
-    unsigned long timestamp;
     int screen;
     int xinerama;
     WId launched_by;
@@ -150,7 +141,12 @@ public:
     void got_remove_startup_info(const QString &msg_P);
     void new_startup_info_internal(const KStartupInfoId &id_P,
                                    Data &data_P, bool update_only_P);
-    void remove_startup_info_internal(const KStartupInfoId &id_P);
+    void removeAllStartupInfoInternal(const KStartupInfoId &id_P);
+    /**
+     * Emits the gotRemoveStartup signal and erases the @p it from the startups map.
+     * @returns Iterator to next item in the startups map.
+     **/
+    QMap< KStartupInfoId, Data >::iterator removeStartupInfoInternal(QMap< KStartupInfoId, Data >::iterator it);
     void remove_startup_pids(const KStartupInfoId &id, const KStartupInfoData &data);
     void remove_startup_pids(const KStartupInfoData &data);
     startup_t check_startup_internal(WId w, KStartupInfoId *id, KStartupInfoData *data);
@@ -160,7 +156,6 @@ public:
                   KStartupInfoData *data_O);
     bool find_wclass(const QByteArray &res_name_P, const QByteArray &res_class_P,
                      KStartupInfoId *id_O, KStartupInfoData *data_O);
-    static QByteArray get_window_hostname(WId w_P);
     void startups_cleanup_internal(bool age_P);
     void clean_all_noncompliant();
     static QString check_required_startup_fields(const QString &msg,
@@ -185,6 +180,7 @@ public:
 #if KWINDOWSYSTEM_HAVE_X11
           msgs(NET_STARTUP_MSG),
 #endif
+          cleanup(Q_NULLPTR),
           flags(flags_P)
     {
     }
@@ -218,12 +214,14 @@ KStartupInfo::KStartupInfo(int flags_P, QObject *parent_P)
     d->createConnections();
 }
 
+#ifndef KWINDOWSYSTEM_NO_DEPRECATED
 KStartupInfo::KStartupInfo(bool clean_on_cantdetect_P, QObject *parent_P)
     : QObject(parent_P),
       d(new Private(clean_on_cantdetect_P ? CleanOnCantDetect : 0, this))
 {
     d->createConnections();
 }
+#endif
 
 KStartupInfo::~KStartupInfo()
 {
@@ -311,7 +309,7 @@ void KStartupInfo::Private::window_added(WId w_P)
 void KStartupInfo::Private::got_startup_info(const QString &msg_P, bool update_P)
 {
     KStartupInfoId id(msg_P);
-    if (id.none()) {
+    if (id.isNull()) {
         return;
     }
     KStartupInfo::Data data(msg_P);
@@ -321,7 +319,7 @@ void KStartupInfo::Private::got_startup_info(const QString &msg_P, bool update_P
 void KStartupInfo::Private::new_startup_info_internal(const KStartupInfoId &id_P,
         KStartupInfo::Data &data_P, bool update_P)
 {
-    if (id_P.none()) {
+    if (id_P.isNull()) {
         return;
     }
     if (startups.contains(id_P)) {
@@ -384,30 +382,40 @@ void KStartupInfo::Private::got_remove_startup_info(const QString &msg_P)
     KStartupInfoId id(msg_P);
     KStartupInfoData data(msg_P);
     if (data.pids().count() > 0) {
-        if (!id.none()) {
+        if (!id.isNull()) {
             remove_startup_pids(id, data);
         } else {
             remove_startup_pids(data);
         }
         return;
     }
-    remove_startup_info_internal(id);
+    removeAllStartupInfoInternal(id);
 }
 
-void KStartupInfo::Private::remove_startup_info_internal(const KStartupInfoId &id_P)
+void KStartupInfo::Private::removeAllStartupInfoInternal(const KStartupInfoId &id_P)
 {
-    if (startups.contains(id_P)) {
+    auto it = startups.find(id_P);
+    if (it != startups.end()) {
         //qDebug() << "removing";
-        emit q->gotRemoveStartup(id_P, startups[ id_P ]);
-        startups.remove(id_P);
-    } else if (silent_startups.contains(id_P)) {
-        //qDebug() << "removing silent";
-        silent_startups.remove(id_P);
-    } else if (uninited_startups.contains(id_P)) {
-        //qDebug() << "removing uninited";
-        uninited_startups.remove(id_P);
+        emit q->gotRemoveStartup(it.key(), it.value());
+        startups.erase(it);
+        return;
     }
-    return;
+    it = silent_startups.find(id_P);
+    if (it != silent_startups.end()) {
+        silent_startups.erase(it);
+        return;
+    }
+    it = uninited_startups.find(id_P);
+    if (it != uninited_startups.end()) {
+        uninited_startups.erase(it);
+    }
+}
+
+QMap< KStartupInfoId, KStartupInfo::Data >::iterator KStartupInfo::Private::removeStartupInfoInternal(QMap< KStartupInfoId, Data >::iterator it)
+{
+    emit q->gotRemoveStartup(it.key(), it.value());
+    return startups.erase(it);
 }
 
 void KStartupInfo::Private::remove_startup_pids(const KStartupInfoData &data_P)
@@ -449,13 +457,13 @@ void KStartupInfo::Private::remove_startup_pids(const KStartupInfoId &id_P,
         data->d->remove_pid(*it2);    // remove all pids from the info
     }
     if (data->pids().count() == 0) { // all pids removed -> remove info
-        remove_startup_info_internal(id_P);
+        removeAllStartupInfoInternal(id_P);
     }
 }
 
 bool KStartupInfo::sendStartup(const KStartupInfoId &id_P, const KStartupInfoData &data_P)
 {
-    if (id_P.none()) {
+    if (id_P.isNull()) {
         return false;
     }
 #if  KWINDOWSYSTEM_HAVE_X11
@@ -474,7 +482,7 @@ bool KStartupInfo::sendStartup(const KStartupInfoId &id_P, const KStartupInfoDat
 bool KStartupInfo::sendStartupX(Display *disp_P, const KStartupInfoId &id_P,
                                 const KStartupInfoData &data_P)
 {
-    if (id_P.none()) {
+    if (id_P.isNull()) {
         return false;
     }
 #if KWINDOWSYSTEM_HAVE_X11
@@ -512,7 +520,7 @@ QString KStartupInfo::Private::check_required_startup_fields(const QString &msg,
 
 bool KStartupInfo::sendChange(const KStartupInfoId &id_P, const KStartupInfoData &data_P)
 {
-    if (id_P.none()) {
+    if (id_P.isNull()) {
         return false;
     }
 #if KWINDOWSYSTEM_HAVE_X11
@@ -530,7 +538,7 @@ bool KStartupInfo::sendChange(const KStartupInfoId &id_P, const KStartupInfoData
 bool KStartupInfo::sendChangeX(Display *disp_P, const KStartupInfoId &id_P,
                                const KStartupInfoData &data_P)
 {
-    if (id_P.none()) {
+    if (id_P.isNull()) {
         return false;
     }
 #if KWINDOWSYSTEM_HAVE_X11
@@ -549,7 +557,7 @@ bool KStartupInfo::sendChangeX(Display *disp_P, const KStartupInfoId &id_P,
 
 bool KStartupInfo::sendFinish(const KStartupInfoId &id_P)
 {
-    if (id_P.none()) {
+    if (id_P.isNull()) {
         return false;
     }
 #if KWINDOWSYSTEM_HAVE_X11
@@ -563,7 +571,7 @@ bool KStartupInfo::sendFinish(const KStartupInfoId &id_P)
 
 bool KStartupInfo::sendFinishX(Display *disp_P, const KStartupInfoId &id_P)
 {
-    if (id_P.none()) {
+    if (id_P.isNull()) {
         return false;
     }
 #if KWINDOWSYSTEM_HAVE_X11
@@ -580,7 +588,7 @@ bool KStartupInfo::sendFinishX(Display *disp_P, const KStartupInfoId &id_P)
 
 bool KStartupInfo::sendFinish(const KStartupInfoId &id_P, const KStartupInfoData &data_P)
 {
-//    if( id_P.none()) // id may be none, the pids and hostname matter then
+//    if( id_P.isNull()) // id may be null, the pids and hostname matter then
 //        return false;
 #if KWINDOWSYSTEM_HAVE_X11
     KXMessages msgs;
@@ -598,7 +606,7 @@ bool KStartupInfo::sendFinish(const KStartupInfoId &id_P, const KStartupInfoData
 bool KStartupInfo::sendFinishX(Display *disp_P, const KStartupInfoId &id_P,
                                const KStartupInfoData &data_P)
 {
-//    if( id_P.none()) // id may be none, the pids and hostname matter then
+//    if( id_P.isNull()) // id may be null, the pids and hostname matter then
 //        return false;
 #if KWINDOWSYSTEM_HAVE_X11
     QString msg = QString::fromLatin1("remove: %1 %2")
@@ -625,7 +633,7 @@ void KStartupInfo::appStarted(const QByteArray &startup_id)
 {
     KStartupInfoId id;
     id.initId(startup_id);
-    if (id.none()) {
+    if (id.isNull()) {
         return;
     }
 #if KWINDOWSYSTEM_HAVE_X11
@@ -639,16 +647,11 @@ void KStartupInfo::appStarted(const QByteArray &startup_id)
 #endif
 }
 
-void KStartupInfo::disableAutoAppStartedSending(bool disable)
-{
-    auto_app_started_sending = !disable;
-}
-
 void KStartupInfo::silenceStartup(bool silence)
 {
     KStartupInfoId id;
     id.initId(startupId());
-    if (id.none()) {
+    if (id.isNull()) {
         return;
     }
     KStartupInfoData data;
@@ -656,13 +659,6 @@ void KStartupInfo::silenceStartup(bool silence)
     sendChange(id, data);
 }
 
-void KStartupInfo::handleAutoAppStartedSending()
-{
-    if (auto_app_started_sending) {
-        appStarted();
-    }
-}
-
 QByteArray KStartupInfo::startupId()
 {
     if (s_startup_id.isEmpty()) {
@@ -679,9 +675,6 @@ void KStartupInfo::setStartupId(const QByteArray &startup_id)
     if (startup_id == startupId()) {
         return;
     }
-#if KWINDOWSYSTEM_HAVE_X11
-    KStartupInfo::handleAutoAppStartedSending(); // finish old startup notification if needed
-#endif
     if (startup_id.isEmpty()) {
         s_startup_id = "0";
     } else {
@@ -732,7 +725,6 @@ void KStartupInfo::setNewStartupId(QWidget *window, const QByteArray &startup_id
     Q_UNUSED(activate)
     Q_UNUSED(window)
 #endif
-    KStartupInfo::handleAutoAppStartedSending();
 }
 
 KStartupInfo::startup_t KStartupInfo::checkStartup(WId w_P, KStartupInfoId &id_O,
@@ -784,26 +776,19 @@ KStartupInfo::startup_t KStartupInfo::Private::check_startup_internal(WId w_P, K
         return CantDetect;
     }
     NETWinInfo info(QX11Info::connection(),  w_P, QX11Info::appRootWindow(),
-                    NET::WMWindowType | NET::WMPid | NET::WMState);
+                    NET::WMWindowType | NET::WMPid | NET::WMState,
+                    NET::WM2WindowClass | NET::WM2ClientMachine | NET::WM2TransientFor);
     pid_t pid = info.pid();
     if (pid > 0) {
-        QByteArray hostname = get_window_hostname(w_P);
+        QByteArray hostname = info.clientMachine();
         if (!hostname.isEmpty()
                 && find_pid(pid, hostname, id_O, data_O)) {
             return Match;
         }
         // try XClass matching , this PID stuff sucks :(
     }
-    XClassHint hint;
-    if (XGetClassHint(QX11Info::display(), w_P, &hint) != 0) {
-        // We managed to read the class hint
-        QByteArray res_name = hint.res_name;
-        QByteArray res_class = hint.res_class;
-        XFree(hint.res_name);
-        XFree(hint.res_class);
-        if (find_wclass(res_name, res_class, id_O, data_O)) {
-            return Match;
-        }
+    if (find_wclass(info.windowClassName(), info.windowClassClass(), id_O, data_O)) {
+        return Match;
     }
     // ignore NET::Tool and other special window types, if they can't be matched
     NET::WindowType type = info.windowType(NET::NormalMask | NET::DesktopMask
@@ -819,10 +804,8 @@ KStartupInfo::startup_t KStartupInfo::Private::check_startup_internal(WId w_P, K
         return NoMatch;
     }
     // lets see if this is a transient
-    Window transient_for;
-    if (XGetTransientForHint(QX11Info::display(), static_cast< Window >(w_P), &transient_for)
-            && static_cast< WId >(transient_for) != QX11Info::appRootWindow()
-            && transient_for != None) {
+    xcb_window_t transient_for = info.transientFor();
+    if (transient_for != QX11Info::appRootWindow() && transient_for != XCB_WINDOW_NONE) {
         return NoMatch;
     }
 #endif
@@ -865,7 +848,7 @@ bool KStartupInfo::Private::find_pid(pid_t pid_P, const QByteArray &hostname_P,
                 *data_O = *it;
             }
             // non-compliant, remove on first match
-            remove_startup_info_internal(it.key());
+            removeStartupInfoInternal(it);
             //qDebug() << "check_startup_pid:match";
             return true;
         }
@@ -892,7 +875,7 @@ bool KStartupInfo::Private::find_wclass(const QByteArray &_res_name, const QByte
                 *data_O = *it;
             }
             // non-compliant, remove on first match
-            remove_startup_info_internal(it.key());
+            removeStartupInfoInternal(it);
             //qDebug() << "check_startup_wclass:match";
             return true;
         }
@@ -900,53 +883,18 @@ bool KStartupInfo::Private::find_wclass(const QByteArray &_res_name, const QByte
     return false;
 }
 
-#if KWINDOWSYSTEM_HAVE_X11
-static Atom net_startup_atom = None;
-
-static QByteArray read_startup_id_property(WId w_P)
-{
-    QByteArray ret;
-    unsigned char *name_ret;
-    Atom type_ret;
-    int format_ret;
-    unsigned long nitems_ret = 0, after_ret = 0;
-    if (XGetWindowProperty(QX11Info::display(), w_P, net_startup_atom, 0l, 4096,
-                           False, utf8_string_atom, &type_ret, &format_ret, &nitems_ret, &after_ret, &name_ret)
-            == Success) {
-        if (type_ret == utf8_string_atom && format_ret == 8 && name_ret != NULL) {
-            ret = reinterpret_cast< char * >(name_ret);
-        }
-        if (name_ret != NULL) {
-            XFree(name_ret);
-        }
-    }
-    return ret;
-}
-
-#endif
-
 QByteArray KStartupInfo::windowStartupId(WId w_P)
 {
 #if KWINDOWSYSTEM_HAVE_X11
     if (!QX11Info::isPlatformX11()) {
         return QByteArray();
     }
-    if (net_startup_atom == None) {
-        net_startup_atom = XInternAtom(QX11Info::display(), NET_STARTUP_WINDOW, False);
-    }
-    if (utf8_string_atom == None) {
-        utf8_string_atom = XInternAtom(QX11Info::display(), "UTF8_STRING", False);
-    }
-    QByteArray ret = read_startup_id_property(w_P);
-    if (ret.isEmpty()) {
+    NETWinInfo info(QX11Info::connection(), w_P, QX11Info::appRootWindow(), 0, NET::WM2StartupId | NET::WM2GroupLeader);
+    QByteArray ret = info.startupId();
+    if (ret.isEmpty() && info.groupLeader() != XCB_WINDOW_NONE) {
         // retry with window group leader, as the spec says
-        XWMHints *hints = XGetWMHints(QX11Info::display(), w_P);
-        if (hints && (hints->flags & WindowGroupHint) != 0) {
-            ret = read_startup_id_property(hints->window_group);
-        }
-        if (hints) {
-            XFree(hints);
-        }
+        NETWinInfo groupLeaderInfo(QX11Info::connection(), info.groupLeader(), QX11Info::appRootWindow(), 0, NET::WM2StartupId);
+        ret = groupLeaderInfo.startupId();
     }
     return ret;
 #else
@@ -964,45 +912,14 @@ void KStartupInfo::setWindowStartupId(WId w_P, const QByteArray &id_P)
     if (id_P.isNull()) {
         return;
     }
-    if (net_startup_atom == None) {
-        net_startup_atom = XInternAtom(QX11Info::display(), NET_STARTUP_WINDOW, False);
-    }
-    if (utf8_string_atom == None) {
-        utf8_string_atom = XInternAtom(QX11Info::display(), "UTF8_STRING", False);
-    }
-    XChangeProperty(QX11Info::display(), w_P, net_startup_atom, utf8_string_atom, 8,
-                    PropModeReplace, reinterpret_cast< const unsigned char * >(id_P.data()), id_P.length());
+    NETWinInfo info(QX11Info::connection(), w_P, QX11Info::appRootWindow(), 0, 0);
+    info.setStartupId(id_P.constData());
 #else
     Q_UNUSED(w_P)
     Q_UNUSED(id_P)
 #endif
 }
 
-QByteArray KStartupInfo::Private::get_window_hostname(WId w_P)
-{
-#if KWINDOWSYSTEM_HAVE_X11
-    if (!QX11Info::isPlatformX11()) {
-        return QByteArray();
-    }
-    XTextProperty tp;
-    char **hh;
-    int cnt;
-    if (XGetWMClientMachine(QX11Info::display(), w_P, &tp) != 0
-            && XTextPropertyToStringList(&tp, &hh, &cnt) != 0) {
-        if (cnt == 1) {
-            QByteArray hostname = hh[ 0 ];
-            XFreeStringList(hh);
-            return hostname;
-        }
-        XFreeStringList(hh);
-    }
-#else
-    Q_UNUSED(w_P)
-#endif
-    // no hostname
-    return QByteArray();
-}
-
 void KStartupInfo::setTimeout(unsigned int secs_P)
 {
     d->timeout = secs_P;
@@ -1066,14 +983,22 @@ void KStartupInfo::Private::clean_all_noncompliant()
             ++it;
             continue;
         }
-        const KStartupInfoId &key = it.key();
-        ++it;
-        //qDebug() << "entry cleaning:" << key.id();
-        remove_startup_info_internal(key);
+        it = removeStartupInfoInternal(it);
     }
 }
 
 QByteArray KStartupInfo::createNewStartupId()
+{
+    quint32 timestamp = 0;
+#if KWINDOWSYSTEM_HAVE_X11
+    if (QX11Info::isPlatformX11()) {
+        timestamp = QX11Info::getTimestamp();
+    }
+#endif
+    return KStartupInfo::createNewStartupIdForTimestamp(timestamp);
+}
+
+QByteArray KStartupInfo::createNewStartupIdForTimestamp(quint32 timestamp)
 {
     // Assign a unique id, use hostname+time+pid, that should be 200% unique.
     // Also append the user timestamp (for focus stealing prevention).
@@ -1092,13 +1017,8 @@ QByteArray KStartupInfo::createNewStartupId()
     if (!gethostname(hostname, 255)) {
         hostname[sizeof(hostname) - 1] = '\0';
     }
-#if KWINDOWSYSTEM_HAVE_X11
-    unsigned long qt_x_user_time = QX11Info::appUserTime();
-#else
-    unsigned long qt_x_user_time = 0;
-#endif
     QByteArray id = QString::fromLatin1("%1;%2;%3;%4_TIME%5").arg(hostname).arg(tm.tv_sec)
-                    .arg(tm.tv_usec).arg(getpid()).arg(qt_x_user_time).toUtf8();
+                    .arg(tm.tv_usec).arg(getpid()).arg(timestamp).toUtf8();
     //qDebug() << "creating: " << id << ":" << (qApp ? qAppName() : QString("unnamed app") /* e.g. kdeinit */);
     return id;
 }
@@ -1149,7 +1069,7 @@ void KStartupInfoId::initId(const QByteArray &id_P)
 
 bool KStartupInfoId::setupStartupEnv() const
 {
-    if (none()) {
+    if (isNull()) {
         qunsetenv(NET_STARTUP_ENV);
         return false;
     }
@@ -1211,17 +1131,17 @@ bool KStartupInfoId::operator<(const KStartupInfoId &id_P) const
     return id() < id_P.id();
 }
 
-// KDE5 TODO: rename to isNull ?
-bool KStartupInfoId::none() const
+bool KStartupInfoId::isNull() const
 {
     return d->id.isEmpty() || d->id == "0";
 }
 
 unsigned long KStartupInfoId::timestamp() const
 {
-    if (none()) {
+    if (isNull()) {
         return 0;
     }
+    // As per the spec, the ID must contain the _TIME followed by the timestamp
     int pos = d->id.lastIndexOf("_TIME");
     if (pos >= 0) {
         bool ok;
@@ -1233,25 +1153,6 @@ unsigned long KStartupInfoId::timestamp() const
             return time;
         }
     }
-    // libstartup-notification style :
-    // qsnprintf (s, len, "%s/%s/%lu/%d-%d-%s",
-    //   canonicalized_launcher, canonicalized_launchee, (unsigned long) timestamp,
-    //  (int) getpid (), (int) sequence_number, hostbuf);
-    int pos1 = d->id.lastIndexOf('/');
-    if (pos1 > 0) {
-        int pos2 = d->id.lastIndexOf('/', pos1 - 1);
-        if (pos2 >= 0) {
-            bool ok;
-            unsigned long time = QString(d->id.mid(pos2 + 1, pos1 - pos2 - 1)).toULong(&ok);
-            if (!ok && d->id[ pos2 + 1 ] == '-') {
-                time = QString(d->id.mid(pos2 + 1, pos1 - pos2 - 1)).toLong(&ok);
-            }
-            if (ok) {
-                return time;
-            }
-        }
-    }
-    // bah ... old KStartupInfo or a problem
     return 0;
 }
 
@@ -1287,9 +1188,6 @@ QString KStartupInfoData::Private::to_text() const
     if (silent != KStartupInfoData::Unknown) {
         ret += QString::fromLatin1(" SILENT=%1").arg(silent == KStartupInfoData::Yes ? 1 : 0);
     }
-    if (timestamp != ~0U) {
-        ret += QString::fromLatin1(" TIMESTAMP=%1").arg(timestamp);
-    }
     if (screen != -1) {
         ret += QString::fromLatin1(" SCREEN=%1").arg(screen);
     }
@@ -1314,8 +1212,8 @@ KStartupInfoData::KStartupInfoData(const QString &txt_P) : d(new Private)
     const QString icon_str = QString::fromLatin1("ICON=");
     const QString desktop_str = QString::fromLatin1("DESKTOP=");
     const QString wmclass_str = QString::fromLatin1("WMCLASS=");
-    const QString hostname_str = QString::fromLatin1("HOSTNAME=");   // SELI nonstd
-    const QString pid_str = QString::fromLatin1("PID=");    // SELI nonstd
+    const QString hostname_str = QString::fromLatin1("HOSTNAME="); // added to version 1 (2014)
+    const QString pid_str = QString::fromLatin1("PID=");           // added to version 1 (2014)
     const QString silent_str = QString::fromLatin1("SILENT=");
     const QString timestamp_str = QString::fromLatin1("TIMESTAMP=");
     const QString screen_str = QString::fromLatin1("SCREEN=");
@@ -1345,8 +1243,6 @@ KStartupInfoData::KStartupInfoData(const QString &txt_P) : d(new Private)
             addPid(get_num(*it));
         } else if ((*it).startsWith(silent_str)) {
             d->silent = get_num(*it) != 0 ? Yes : No;
-        } else if ((*it).startsWith(timestamp_str)) {
-            d->timestamp = get_unum(*it);
         } else if ((*it).startsWith(screen_str)) {
             d->screen = get_num(*it);
         } else if ((*it).startsWith(xinerama_str)) {
@@ -1403,9 +1299,6 @@ void KStartupInfoData::update(const KStartupInfoData &data_P)
     if (data_P.silent() != Unknown) {
         d->silent = data_P.silent();
     }
-    if (data_P.timestamp() != ~0U && timestamp() == ~0U) { // don't overwrite
-        d->timestamp = data_P.timestamp();
-    }
     if (data_P.screen() != -1) {
         d->screen = data_P.screen();
     }
@@ -1572,16 +1465,6 @@ KStartupInfoData::TriState KStartupInfoData::silent() const
     return d->silent;
 }
 
-void KStartupInfoData::setTimestamp(unsigned long time)
-{
-    d->timestamp = time;
-}
-
-unsigned long KStartupInfoData::timestamp() const
-{
-    return d->timestamp;
-}
-
 void KStartupInfoData::setScreen(int _screen)
 {
     d->screen = _screen;
@@ -1621,7 +1504,7 @@ void KStartupInfoData::setApplicationId(const QString &desktop)
     // the spec requires this is always a full path, in order for everyone to be able to find it
     QString desk = QStandardPaths::locate(QStandardPaths::ApplicationsLocation, desktop);
     if (desk.isEmpty()) {
-        desk = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kde5/services/" + desktop);
+        desk = QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kservices5/" + desktop);
     }
     if (desk.isEmpty()) {
         return;
@@ -1641,13 +1524,6 @@ long get_num(const QString &item_P)
     return item_P.mid(pos + 1).toLong();
 }
 
-static
-unsigned long get_unum(const QString &item_P)
-{
-    unsigned int pos = item_P.indexOf(QLatin1Char('='));
-    return item_P.mid(pos + 1).toULong();
-}
-
 static
 QString get_str(const QString &item_P)
 {
